@@ -462,7 +462,8 @@ class Interpreter:
         Args:
             message: message storing text to process;
         Returns:
-            message: it contains the tokens and features which are the output of the NLU pipeline;
+            message: it contains the tokens and features which are the output of the
+            NLU pipeline;
         """
 
         for component in self.pipeline:
